@@ -9,8 +9,10 @@ void init_core(py::module &);
 void init_fragment(py::module &);
 // void init_query_condition(py::module &);
 void init_schema_evolution(py::module &);
+#if defined(TILEDB_SERIALIZATION)
 void init_serialization(py::module &);
 void init_test_serialization(py::module &);
+#endif
 void init_test_metadata(py::module &);
 
 PYBIND11_MODULE(main, m) {
@@ -19,8 +21,10 @@ PYBIND11_MODULE(main, m) {
   init_fragment(m);
   //_query_condition(m);
   init_schema_evolution(m);
+#if defined(TILEDB_SERIALIZATION)
   init_serialization(m);
   init_test_serialization(m);
+#endif
   init_test_metadata(m);
 }
 
