@@ -19,19 +19,21 @@ from tiledb.multirange_indexing import getitem_ranges, mr_dense_result_shape
 from tiledb.tests.common import (
     DiskTestCase,
     assert_tail_equal,
+    has_pandas,
+    has_pyarrow,
     intspace,
     SUPPORTED_DATETIME64_DTYPES,
     rand_datetime64_array,
-    assert_all_arrays_equal,
+    assert_dict_arrays_equal,
 )
 
 import hypothesis.extra.numpy as npst
 
 
-def make_1d_dense(path, attr_name="", attr_dtype=np.int64):
+def make_1d_dense(path, attr_name="", attr_dtype=np.int64, dim_dtype=np.uint64):
     a_orig = np.arange(36)
 
-    dom = tiledb.Domain(tiledb.Dim(domain=(0, 35), tile=35, dtype=np.uint64))
+    dom = tiledb.Domain(tiledb.Dim(domain=(0, 35), tile=35, dtype=dim_dtype))
     att = tiledb.Attr(name=attr_name, dtype=attr_dtype)
     schema = tiledb.ArraySchema(domain=dom, attrs=(att,), sparse=False)
     tiledb.DenseArray.create(path, schema)
@@ -138,6 +140,73 @@ class TestMultiRangeAuxiliary(DiskTestCase):
 
 
 class TestMultiRange(DiskTestCase):
+    @pytest.mark.skipif(
+        not has_pyarrow() or not has_pandas(),
+        reason="pyarrow and/or pandas not installed",
+    )
+    def test_return_arrow_indexers(self):
+        uri = self.path("multirange_behavior_sparse")
+
+        schema = tiledb.ArraySchema(
+            domain=tiledb.Domain(
+                tiledb.Dim(name="idx", domain=(-5, 5), dtype=np.int64)
+            ),
+            attrs=[tiledb.Attr(name="data", dtype=np.int64)],
+        )
+        tiledb.Array.create(uri, schema)
+        data = np.random.randint(-10, 10, size=11)
+
+        with tiledb.open(uri, "w") as A:
+            A[:] = data
+
+        with tiledb.open(uri, "r") as A:
+            with self.assertRaisesRegex(
+                tiledb.TileDBError,
+                "Cannot initialize return_arrow with use_arrow=False",
+            ):
+                q = A.query(return_arrow=True, use_arrow=False)
+
+            q = A.query(return_arrow=True)
+
+            with self.assertRaisesRegex(
+                tiledb.TileDBError,
+                "`return_arrow=True` requires .df indexer",
+            ):
+                q[:]
+
+            with self.assertRaisesRegex(
+                tiledb.TileDBError,
+                "`return_arrow=True` requires .df indexer",
+            ):
+                q.multi_index[:]
+
+            assert_array_equal(q.df[:]["data"], data)
+
+    @pytest.mark.skipif(
+        not has_pyarrow() or not has_pandas(),
+        reason="pyarrow and/or pandas not installed",
+    )
+    @pytest.mark.parametrize("sparse", [True, False])
+    def test_return_large_arrow_table(self, sparse):
+        num = 2**16 - 1
+        uri = self.path("test_return_large_arrow_table")
+        dom = tiledb.Domain(tiledb.Dim(domain=(0, num - 1), dtype=np.uint16))
+        att = tiledb.Attr(dtype=np.uint64)
+        schema = tiledb.ArraySchema(domain=dom, attrs=(att,), sparse=sparse)
+        tiledb.Array.create(uri, schema)
+
+        expected_data = np.arange(num)
+
+        with tiledb.open(uri, "w") as A:
+            if sparse:
+                A[np.arange(num)] = expected_data
+            else:
+                A[:] = expected_data
+
+        with tiledb.open(uri, "r") as arr:
+            actual_data = arr.query(return_arrow=True).df[:]
+            assert_array_equal(actual_data[:][""], expected_data)
+
     def test_multirange_behavior(self):
         uri = self.path("multirange_behavior_sparse")
 
@@ -662,6 +731,7 @@ class TestMultiRange(DiskTestCase):
                 np.array([], dtype=np.uint64),
             )
 
+    @pytest.mark.skipif(not has_pandas(), reason="pandas not installed")
     def test_fixed_multi_attr_df(self):
         uri = self.path("test_fixed_multi_attr_df")
         dom = tiledb.Domain(
@@ -695,6 +765,7 @@ class TestMultiRange(DiskTestCase):
             result = A.query(attrs=["111"], use_arrow=False)
             assert_array_equal(result.df[0]["111"], data_111)
 
+    @pytest.mark.skipif(not has_pandas(), reason="pandas not installed")
     def test_var_multi_attr_df(self):
         uri = self.path("test_var_multi_attr_df")
         dom = tiledb.Domain(
@@ -776,6 +847,7 @@ class TestMultiRange(DiskTestCase):
             assert A.nonempty_domain() is None
             assert_array_equal(A.multi_index[:][""], A[:][""])
 
+    @pytest.mark.skipif(not has_pandas(), reason="pandas not installed")
     def test_multi_index_query_args(self):
         uri = self.path("test_multi_index_query_args")
         schema = tiledb.ArraySchema(
@@ -801,6 +873,7 @@ class TestMultiRange(DiskTestCase):
             assert_array_equal(q.multi_index[:]["a"], q.df[:]["a"])
             assert all(q[:]["a"] >= 5)
 
+    @pytest.mark.skipif(not has_pandas(), reason="pandas not installed")
     def test_multi_index_timing(self):
         path = self.path("test_multi_index_timing")
         attr_name = "a"
@@ -814,3 +887,113 @@ class TestMultiRange(DiskTestCase):
             assert "py.getitem_time.buffer_conversion_time :" in internal_stats
             assert "py.getitem_time.pandas_index_update_time :" in internal_stats
         tiledb.stats_disable()
+
+
+# parametrize dtype and sparse
+@pytest.mark.parametrize(
+    "dim_dtype",
+    [
+        np.int64,
+        np.uint64,
+        np.int32,
+        np.uint32,
+        np.int16,
+        np.uint16,
+        np.int8,
+        np.uint8,
+    ],
+)
+class TestMultiIndexND(DiskTestCase):
+    def test_multi_index_ndarray(self, dim_dtype):
+        # TODO support for dense?
+        sparse = True  # ndarray indexing currently only supported for sparse
+
+        path = self.path(f"test_multi_index_ndarray")
+
+        ncells = 10
+        data = np.arange(ncells - 1)
+        coords = np.arange(ncells - 1)
+
+        # use negative range for sparse
+        if sparse and np.issubdtype(dim_dtype, np.signedinteger):
+            coords -= 4
+
+        dom = tiledb.Domain(
+            tiledb.Dim(domain=(coords.min(), coords.max()), dtype=dim_dtype)
+        )
+        att = tiledb.Attr(dtype=np.int8)
+        schema = tiledb.ArraySchema(domain=dom, attrs=(att,), sparse=sparse)
+        tiledb.Array.create(path, schema)
+
+        with tiledb.open(path, "w") as A:
+            if sparse:
+                A[coords] = data
+            else:
+                A[:] = data
+
+        with tiledb.open(path) as A:
+            assert_dict_arrays_equal(
+                A.multi_index[coords.tolist()], A.multi_index[coords]
+            )
+            assert_dict_arrays_equal(
+                A.multi_index[coords.tolist()], A.multi_index[coords]
+            )
+
+    def test_multi_index_ndarray_2d(self, dim_dtype):
+        sparse = False
+
+        path = self.path(f"test_multi_index_ndarray_2d")
+
+        ncells = 10
+        ext = ncells - 1
+        if sparse:
+            data = np.arange(ext)
+        else:
+            data = np.arange(ext**2).reshape(ext, ext)
+        d1_coords = np.arange(ext)
+        d2_coords = np.arange(ext, 0, -1)
+
+        # use negative range for sparse
+        if sparse and np.issubdtype(dim_dtype, np.signedinteger):
+            d1_coords -= 4
+
+        d1 = tiledb.Dim(
+            name="d1", domain=(d1_coords.min(), d1_coords.max()), dtype=dim_dtype
+        )
+        d2 = tiledb.Dim(
+            name="d2", domain=(d2_coords.min(), d2_coords.max()), dtype=dim_dtype
+        )
+        dom = tiledb.Domain([d1, d2])
+
+        att = tiledb.Attr(dtype=np.int8)
+        schema = tiledb.ArraySchema(domain=dom, attrs=(att,), sparse=sparse)
+        tiledb.Array.create(path, schema)
+
+        with tiledb.open(path, "w") as A:
+            if sparse:
+                A[d1_coords.tolist(), d2_coords.tolist()] = {"": data}
+            else:
+                A[:] = data
+                # raise ValueError("Test only support sparse")
+
+        with tiledb.open(path) as A:
+            assert_dict_arrays_equal(
+                A.multi_index[d1_coords.tolist(), d2_coords.tolist()],
+                A.multi_index[d1_coords, d2_coords],
+            )
+
+            # note: np.flip below because coords are in reverse order, which is how
+            #       tiledb will return the results for the first query, but not second
+            assert_dict_arrays_equal(
+                A.multi_index[d1_coords.tolist(), np.flip(d2_coords.tolist())],
+                A.multi_index[d1_coords, :],
+            )
+
+            slc = slice(0, ncells - 1, 2)
+            assert_dict_arrays_equal(
+                A.multi_index[d1_coords[slc].tolist(), :],
+                A.multi_index[d1_coords[slc], :],
+            )
+            assert_dict_arrays_equal(
+                A.multi_index[:, d2_coords[slc]], A.multi_index[:, d2_coords[slc]]
+            )
